/*  goopatch.c
 *
 *  Based on GooCanvas Demo
 *  Copyright (C) 2006 - 2010 Damon Chaplin, Alexis Vasic
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "goocanvas.h"
#include <cairo.h>
#include "goopatch.h"

G_DEFINE_TYPE (GooPatch, goo_patch, GOO_TYPE_CANVAS_ITEM_SIMPLE)

static void
goo_patch_init (GooPatch *Patch)
{
    Patch->nIN = 0;
    Patch->nOUT = 0;
    Patch->ID = NULL;
    Patch->PatchName = NULL;
    Patch->x = 0.0;
    Patch->y = 0.0;
    Patch->width = 0.0;
    Patch->height = 0.0;
}

GooPatch*
goo_patch_new (GooCanvasItem	*parent,
		gint		nIN,
		gint		nOUT,
		gchar		*ID,
		gchar		*PatchName,
		gdouble		x,
		gdouble		y
		)
{
    GooPatch		*Patch;
    GooCanvasItem	*Item;
    /* const char		*first_property; */
    /* va_list		var_args; */

    Item = g_object_new (GOO_TYPE_PATCH, NULL);

    Patch = (GooPatch*) Item;
    Patch->nIN = nIN;
    Patch->nOUT = nOUT;
    Patch->ID = ID;
    Patch->PatchName = PatchName;
    Patch->x = x;
    Patch->y = y;

    if (nIN >= nOUT)
    {
	Patch->height = nIN * 20 + 20;
    }
    else
    {
	Patch->height = nOUT * 20 + 20;
    }
    
    Patch->width = 75;

    if (parent)
    {
	Patch->ItemGroup = goo_canvas_group_new (parent, Patch->height, Patch->width, Patch->x, Patch->y, NULL);
	goo_canvas_item_add_child (Patch->ItemGroup, Item, -1);
	g_object_unref (Item);
    }

    return Patch;
}

static void
goo_patch_update (GooCanvasItemSimple	*simple,
		    cairo_t		*cr)
{
    GooPatch *Patch = (GooPatch*) simple;

    /* Compute the new bounds. */
    simple->bounds.x1 = Patch->x;
    simple->bounds.y1 = Patch->y;
    simple->bounds.x2 = Patch->x + Patch->width;
    simple->bounds.y2 = Patch->y + Patch->height;
}

gchar*
goo_patch_get_name (GooPatch *Patch)
{
    return Patch->PatchName;
}

static void
goo_patch_paint (GooCanvasItemSimple		*simple,
		cairo_t				*cr,
		const GooCanvasBounds		*bounds)
{
    GooPatch *Patch = (GooPatch*) simple;

    goo_canvas_rect_new (Patch->ItemGroup,
			Patch->x,
			Patch->y,
			Patch->width,
			Patch->height,
			"stroke-color", "white",
			"line-width", 0.5,
			"fill-color", "DimGray",
			"tooltip", Patch->PatchName, NULL);

    goo_canvas_text_new (Patch->ItemGroup,
			Patch->PatchName,
			Patch->x + Patch->width / 2,
			Patch->y + 15,
			70, GTK_ANCHOR_CENTER,
			"font", "Sans Italic 7",
			"fill_color", "white",
			"alignment", PANGO_ALIGN_CENTER,
			NULL);

    if (Patch->nIN != 0)
    {
	gint i;
	gdouble pINtableheight = Patch->height - 30;
	gdouble pINdistance = 8 + (pINtableheight - (Patch->nIN * 8)) / (Patch->nIN + 1);
	gdouble pINiter = Patch->y + 20 + pINdistance;

	for (i = 1; i <= Patch->nIN; i++)
	{
	    goo_canvas_rect_new (Patch->ItemGroup,
				Patch->x,
				pINiter,
				10,
				8,
				"stroke-color", "white",
				"line-width", 0.5,
				"fill_color", "White",
				"tooltip", "pINessai", NULL);

	    pINiter = pINiter + pINdistance;
	}
    }

    if (Patch->nOUT != 0)
    {
	gint i;
	gdouble pOUTtableheight = Patch->height - 30;
	gdouble pOUTdistance = 8 + (pOUTtableheight - (Patch->nOUT  * 8)) / (Patch->nOUT + 1);
	gdouble pOUTiter = Patch->y + 20 + pOUTdistance;

	for (i = 1; i <= Patch->nOUT; i++)
	{
	    goo_canvas_rect_new (Patch->ItemGroup,
				(Patch->x + Patch->width - 10),
				pOUTiter,
				10,
				8,
				"stroke-color", "white",
				"line-width", 0.5,
				"fill_color", "MintCream",
				"tooltip", "pINessai", NULL);

	    pOUTiter = pOUTiter + pOUTdistance;
	}
    }
}

static gboolean
goo_patch_is_item_at (GooCanvasItemSimple	*simple,
			gdouble              	x,
			gdouble              	y,
			cairo_t             	*cr,
			gboolean             	is_pointer_event)
{
    GooPatch *Patch = (GooPatch*) simple;

    if (x < Patch->x || (x > Patch->x + Patch->width)
	|| y < Patch->y || (y > Patch->y + Patch->height))
    return FALSE;

    return TRUE;
}

static void
goo_patch_class_init (GooPatchClass *klass)
{
    GooCanvasItemSimpleClass *simple_class = (GooCanvasItemSimpleClass*) klass;

    simple_class->simple_update		= goo_patch_update;
    simple_class->simple_paint		= goo_patch_paint;
    simple_class->simple_is_item_at	= goo_patch_is_item_at;
}
